@@ -24,9 +24,12 @@ from pypeit import msgs
 from pypeit import calibrations
 from pypeit.images import buildimage
 from pypeit.display import display
-from pypeit import reduce
+from pypeit import find_objects
+from pypeit import extraction
 from pypeit import spec2dobj
 from pypeit.core import qa
+from pypeit.core import parse
+from pypeit.core import findobj_skymask
 from pypeit import specobjs
 from pypeit.spectrographs.util import load_spectrograph
 from pypeit import slittrace
@@ -35,6 +38,7 @@ from pypeit.history import History
 from pypeit.par.util import parse_pypeit_file
 from pypeit.par import PypeItPar
 from pypeit.metadata import PypeItMetaData
+from pypeit.manual_extract import ManualExtractionObj
 
 from linetools import utils as ltu
 
@@ -90,7 +94,7 @@ class PypeIt:
         self.msgs_reset()
         
         # Load
-        cfg_lines, data_files, frametype, usrdata, setups \
+        cfg_lines, data_files, frametype, usrdata, setups, _ \
                 = parse_pypeit_file(pypeit_file, runtime=True)
         self.calib_only = calib_only
 
@@ -139,6 +143,8 @@ class PypeIt:
         #   - Interpret automated or user-provided data from the PypeIt
         #   file
         self.fitstbl.finalize_usr_build(frametype, setups[0])
+
+        
         # --------------------------------------------------------------
         #   - Write .calib file (For QA naming amongst other things)
         calib_file = pypeit_file.replace('.pypeit', '.calib')
@@ -280,8 +286,9 @@ class PypeIt:
             grp_frames = frame_indx[in_grp]
 
             # Find the detectors to reduce
-            detectors = PypeIt.select_detectors(detnum=self.par['rdx']['detnum'],
-                                                ndet=self.spectrograph.ndet)
+#            detectors = PypeIt.select_detectors(detnum=self.par['rdx']['detnum'],
+#                                                ndet=self.spectrograph.ndet)
+            detectors = self.spectrograph.select_detectors(subset=self.par['rdx']['detnum'])
             calib_dict[calib_grp] = {}
             # Loop on Detectors
             for self.det in detectors:
@@ -289,7 +296,9 @@ class PypeIt:
                 self.caliBrate = calibrations.Calibrations.get_instance(
                     self.fitstbl, self.par['calibrations'], self.spectrograph,
                     self.calibrations_path, qadir=self.qa_path, reuse_masters=self.reuse_masters,
-                    show=self.show, slitspat_num=self.par['rdx']['slitspatnum'])
+                    show=self.show,
+                    user_slits=slittrace.merge_user_slit(self.par['rdx']['slitspatnum'],
+                                                         self.par['rdx']['maskIDs']))
                 # Do it
                 # TODO: Why isn't set_config part of the Calibrations.__init__ method?
                 self.caliBrate.set_config(grp_frames[0], self.det, self.par['calibrations'])
@@ -355,6 +364,7 @@ class PypeIt:
         # Frame indices
         frame_indx = np.arange(len(self.fitstbl))
 
+        # Standard Star(s) Loop
         # Iterate over each calibration group and reduce the standards
         for i in range(self.fitstbl.n_calib_groups):
 
@@ -383,6 +393,7 @@ class PypeIt:
                     msgs.info('Output file: {:s} already exists'.format(self.fitstbl.construct_basename(frames[0])) +
                               '. Set overwrite=True to recreate and overwrite.')
 
+        # Science Frame(s) Loop
         # Iterate over each calibration group again and reduce the science frames
         for i in range(self.fitstbl.n_calib_groups):
             # Find all the frames in this calibration group
@@ -396,6 +407,7 @@ class PypeIt:
             science_basename = [None]*len(grp_science)
             # Loop on unique comb_id
             u_combid = np.unique(self.fitstbl['comb_id'][grp_science])
+        
             for j, comb_id in enumerate(u_combid):
                 frames = np.where(self.fitstbl['comb_id'] == comb_id)[0]
                 # Find all frames whose comb_id matches the current frames bkg_id.
@@ -418,7 +430,8 @@ class PypeIt:
                     science_basename[j] = self.basename
 
                     # TODO come up with sensible naming convention for save_exposure for combined files
-                    self.save_exposure(frames[0], sci_spec2d, sci_sobjs, self.basename, history)
+                    self.save_exposure(frames[0], sci_spec2d, sci_sobjs,
+                                       self.basename, history)
                 else:
                     msgs.warn('Output file: {:s} already exists'.format(self.fitstbl.construct_basename(frames[0])) +
                               '. Set overwrite=True to recreate and overwrite.')
@@ -428,46 +441,13 @@ class PypeIt:
         # Finish
         self.print_end_time()
 
-    # This is a static method to allow for use in coadding script 
-    @staticmethod
-    def select_detectors(detnum=None, ndet=1, slitspatnum=None):
-        """
-        Return the 1-indexed list of detectors to reduce.
-
-        Args:
-            detnum (:obj:`int`, :obj:`list`, optional):
-                One or more detectors to reduce.  If None, return the full list
-                for the provided number of detectors (``ndet``).  Should be None
-                if ``slitspatnum`` is provided.
-            ndet (:obj:`int`, optional):
-                The number of detectors for this instrument.  Only used
-                if ``detnum`` is None.
-            slitspatnum (:obj:`str`, optional):
-                A standard format string used to identify a slit by its detector
-                number and spatial pixel.  Should be None if ``detnum`` is
-                provided.
-
-        Returns:
-            :obj:`list`: List of detectors to be reduced.
-        """
-        if detnum is not None and slitspatnum is not None:
-            msgs.error('You cannot specify both detnum and slitspatnum.  Too painful for '
-                       'over-writing SpecObjs.')
-        if detnum is None and slitspatnum is None:
-            return np.arange(1, ndet+1).tolist()
-        elif detnum is not None:
-            return np.atleast_1d(detnum).tolist()
-        else:
-            return slittrace.parse_slitspatnum(slitspatnum)[0].tolist()
-
-    # TODO: update doc string.  frames can be a list...
     def reduce_exposure(self, frames, bg_frames=None, std_outfile=None):
         """
         Reduce a single exposure
 
         Args:
-            frame (:obj:`int`):
-                0-indexed row in :attr:`fitstbl` with the frame to
+            frames (:obj:`list`):
+                List of 0-indexed rows in :attr:`fitstbl` with the frames to
                 reduce.
             bg_frames (:obj:`list`, optional):
                 List of frame indices for the background.
@@ -481,34 +461,27 @@ class PypeIt:
 
         """
 
-        # TODO:
-        # - change doc string to reflect that more than one frame can be
-        #   provided
-
         # if show is set, clear the ginga channels at the start of each new sci_ID
         if self.show:
             # TODO: Put this in a try/except block?
             display.clear_all()
 
         has_bg = True if bg_frames is not None and len(bg_frames) > 0 else False
-        # Is this an IR reduction?
-        # TODO: Why specific to IR?
-        # JFH This is not specific to IR, but to b/g subtraction with frames. The flag though is self.ir_redux. Perhaps
-        # we should rename this to bg_redux or something like that, since it need not be IR.
+        # Is this an b/g subtraction reduction?
         if has_bg:
-            self.ir_redux = True
+            self.bkg_redux = True
             # The default is to find_negative objects if the bg_frames are classified as "science", and to not find_negative
             # objects if the bg_frames are classified as "sky". This can be explicitly overridden if
             # par['reduce']['findobj']['find_negative'] is set to something other than the default of None.
             self.find_negative = ('science' in self.fitstbl['frametype'][bg_frames[0]]) \
                 if self.par['reduce']['findobj']['find_negative'] is None else self.par['reduce']['findobj']['find_negative']
         else:
-            self.ir_redux = False
+            self.bkg_redux = False
             self.find_negative= False
 
         # Container for all the Spec2DObj
         all_spec2d = spec2dobj.AllSpec2DObj()
-        all_spec2d['meta']['ir_redux'] = self.ir_redux
+        all_spec2d['meta']['bkg_redux'] = self.bkg_redux
         all_spec2d['meta']['find_negative'] = self.find_negative
         # TODO -- Should we reset/regenerate self.slits.mask for a new exposure
 
@@ -516,11 +489,17 @@ class PypeIt:
         all_specobjs_objfind = specobjs.SpecObjs()
         # container for specobjs during second loop (extraction)
         all_specobjs_extract = specobjs.SpecObjs()
-        # list of skymask and global_sky obtained during objfind and used in extraction
-        skymask_list = []
-        global_sky_list = []
+        # list of global_sky obtained during objfind and used in extraction
+        initial_sky_list = []
         # list of sciImg
         sciImg_list = []
+        # List of detectors with successful calibration
+        calibrated_det = []
+        # list of successful MasterSlits calibrations to be used in the extraction loop
+        calib_slits = []
+        # List of objFind objects
+        objFind_list = []
+
 
         # Print status message
         msgs_string = 'Reducing target {:s}'.format(self.fitstbl['target'][frames[0]]) + msgs.newline()
@@ -538,22 +517,16 @@ class PypeIt:
             msgs.info(bg_msgs_string)
 
         # Find the detectors to reduce
-        detectors = PypeIt.select_detectors(detnum=self.par['rdx']['detnum'],
-                                            slitspatnum=self.par['rdx']['slitspatnum'],
-                                            ndet=self.spectrograph.ndet)
-        if len(detectors) != self.spectrograph.ndet:
-            msgs.warn('Not reducing detectors: {0}'.format(' '.join([ str(d) for d in 
-                                set(np.arange(self.spectrograph.ndet)+1)-set(detectors)])))
-
-        # List of detectors with successful calibration
-        calibrated_det = []
-        # list of successful MasterSlits calibrations to be used in the extraction loop
-        calib_slits = []
+        subset = self.par['rdx']['slitspatnum'] if self.par['rdx']['slitspatnum'] is not None \
+                    else self.par['rdx']['detnum']
+        detectors = self.spectrograph.select_detectors(subset=subset)
+        msgs.info(f'Detectors to work on: {detectors}')
 
         # Loop on Detectors
         # TODO: Attempt to put in a multiprocessing call here?
         # objfind
         for self.det in detectors:
+            msgs.info("Working on detector {0}".format(self.det))
             # run calibration
             self.caliBrate = self.calib_one(frames, self.det)
             if not self.caliBrate.success:
@@ -569,34 +542,39 @@ class PypeIt:
             # in the slitmask stuff in between the two loops
             calib_slits.append(self.caliBrate.slits)
             # global_sky, skymask and sciImg are needed in the extract loop
-            global_sky, sobjs_obj, skymask, sciImg = self.objfind_one(frames, self.det, bg_frames,
-                                                                      std_outfile=std_outfile)
+            initial_sky, sobjs_obj, sciImg, objFind = self.objfind_one(frames, self.det, bg_frames,
+                                                                       std_outfile=std_outfile)
             if len(sobjs_obj)>0:
                 all_specobjs_objfind.add_sobj(sobjs_obj)
-            skymask_list.append(skymask)
-            global_sky_list.append(global_sky)
+            initial_sky_list.append(initial_sky)
             sciImg_list.append(sciImg)
+            objFind_list.append(objFind)
 
         # slitmask stuff
         if self.par['reduce']['slitmask']['assign_obj']:
             # get object positions from slitmask design and slitmask offsets for all the detectors
             spat_flexure = np.array([ss.spat_flexure for ss in sciImg_list])
-            platescale = np.array([ss.detector.platescale for ss in sciImg_list])
+            # Grab platescale with binning
+            bin_spec, bin_spat = parse.parse_binning(self.binning)
+            platescale = np.array([ss.detector.platescale*bin_spat for ss in sciImg_list])
             # get the dither offset if available
             if self.par['reduce']['slitmask']['use_dither_offset']:
-                dither = self.spectrograph.parse_dither_pattern([self.fitstbl.frame_paths(frames[0])])
+                dither = self.spectrograph.parse_dither_pattern(
+                    [self.fitstbl.frame_paths(frames[0])])
                 dither_off = dither[2][0] if dither is not None else None
             else:
                 dither_off = None
-            calib_slits = slittrace.get_maskdef_objpos_offset_alldets(all_specobjs_objfind, calib_slits, spat_flexure, platescale,
-                                                                      self.par['calibrations']['slitedges']['det_buffer'],
-                                                                      self.par['reduce']['slitmask'], dither_off=dither_off)
+            calib_slits = slittrace.get_maskdef_objpos_offset_alldets(
+                all_specobjs_objfind, calib_slits, spat_flexure, platescale,
+                self.par['calibrations']['slitedges']['det_buffer'],
+                self.par['reduce']['slitmask'], dither_off=dither_off)
             # determine if slitmask offsets exist and compute an average offsets over all the detectors
-            calib_slits = slittrace.average_maskdef_offset(calib_slits, platescale[0], self.spectrograph.list_detectors())
+            calib_slits = slittrace.average_maskdef_offset(
+                calib_slits, platescale[0], self.spectrograph.list_detectors())
             # slitmask design matching and add undetected objects
-            all_specobjs_objfind = slittrace.assign_addobjs_alldets(all_specobjs_objfind, calib_slits, spat_flexure, platescale,
-                                                                      self.par['reduce']['findobj']['find_fwhm'],
-                                                                      self.par['reduce']['slitmask'])
+            all_specobjs_objfind = slittrace.assign_addobjs_alldets(
+                all_specobjs_objfind, calib_slits, spat_flexure, platescale,
+                self.par['reduce']['slitmask'], self.par['reduce']['findobj']['find_fwhm'])
 
         # Extract
         for i, self.det in enumerate(calibrated_det):
@@ -604,16 +582,20 @@ class PypeIt:
             self.caliBrate = self.calib_one(frames, self.det)
             self.caliBrate.slits = calib_slits[i]
 
+            detname = sciImg_list[i].detector.name
+
             # TODO: pass back the background frame, pass in background
             # files as an argument. extract one takes a file list as an
             # argument and instantiates science within
             if all_specobjs_objfind.nobj > 0:
-                all_specobjs_on_det = all_specobjs_objfind[all_specobjs_objfind.DET == self.det]
+                all_specobjs_on_det = all_specobjs_objfind[all_specobjs_objfind.DET == detname]
             else:
                 all_specobjs_on_det = all_specobjs_objfind
-            all_spec2d[self.det], tmp_sobjs \
-                    = self.extract_one(frames, self.det, sciImg_list[i], global_sky_list[i],
-                                       all_specobjs_on_det, skymask_list[i])
+
+            # Extract
+            all_spec2d[detname], tmp_sobjs \
+                    = self.extract_one(frames, self.det, sciImg_list[i], objFind_list[i],
+                                       initial_sky_list[i], all_specobjs_on_det)
             # Hold em
             if tmp_sobjs.nobj > 0:
                 all_specobjs_extract.add_sobj(tmp_sobjs)
@@ -661,20 +643,31 @@ class PypeIt:
         Returns the trace of the standard if it is applicable to the current reduction
 
         Args:
-            std_redux (bool): If False, proceed
-            det (int): Detector index
-            std_outfile (str): Filename for the standard star spec1d file
+            std_redux (:obj:`bool`):
+                Flag that the current reduction *is* the reduction of a standard
+                and that this step should be skipped.   So, if this is False,
+                the method will proceed; otherwise, the returned value is always
+                None.
+            det (:obj:`int`, :obj:`tuple`):
+                1-indexed detector(s) to process.
+            std_outfile (:obj:`str`):
+                Filename with the standard star spec1d file.  Can be None.
 
         Returns:
-            ndarray or None: Trace of the standard star on input detector
-
+            `numpy.ndarray`_: Trace of the standard star on input detector.
+            Will be None if ``std_redux`` is true, if ``std_outfile`` is None,
+            or if the selected detector/mosaic is not available in the provided
+            spec1d file.
         """
         if std_redux is False and std_outfile is not None:
             sobjs = specobjs.SpecObjs.from_fitsfile(std_outfile)
+            detname = self.spectrograph.get_det_name(det)
             # Does the detector match?
-            # TODO Instrument specific logic here could be implemented with the parset. For example LRIS-B or LRIS-R we
-            # we would use the standard from another detector
-            this_det = sobjs.DET == det
+            # TODO: Instrument specific logic here could be implemented with the
+            # parset. For example LRIS-B or LRIS-R we we would use the standard
+            # from another detector.
+
+            this_det = sobjs.DET == detname
             if np.any(this_det):
                 sobjs_det = sobjs[this_det]
                 sobjs_std = sobjs_det.get_std()
@@ -716,8 +709,12 @@ class PypeIt:
         # Instantiate Calibrations class
         caliBrate = calibrations.Calibrations.get_instance(
             self.fitstbl, self.par['calibrations'], self.spectrograph,
-            self.calibrations_path, qadir=self.qa_path, reuse_masters=self.reuse_masters,
-            show=self.show, slitspat_num=self.par['rdx']['slitspatnum'])
+            self.calibrations_path, qadir=self.qa_path, 
+            reuse_masters=self.reuse_masters,
+            show=self.show, 
+            user_slits=slittrace.merge_user_slit(
+                self.par['rdx']['slitspatnum'], self.par['rdx']['maskIDs']))
+            #slitspat_num=self.par['rdx']['slitspatnum'])
         # These need to be separate to accomodate COADD2D
         caliBrate.set_config(frames[0], det, self.par['calibrations'])
         caliBrate.run_the_steps()
@@ -748,11 +745,11 @@ class PypeIt:
             Initial global sky model
         sobjs_obj : :class:`~pypeit.specobjs.SpecObjs`
             List of objects found
-        skymask : `numpy.ndarray`_
-            Boolean image indicating which pixels are useful for global sky
-            subtraction
         sciImg : :class:`~pypeit.images.pypeitimage.PypeItImage`
             Science image
+        objFind : :class:`~pypeit.find_objects.FindObjects`
+            Object finding object
+
         """
         # Grab some meta-data needed for the reduction from the fitstbl
         self.objtype, self.setup, self.obstime, self.basename, self.binning \
@@ -791,29 +788,34 @@ class PypeIt:
                 slits=self.caliBrate.slits,  # For flexure correction
                 ignore_saturation=False), frame_par['process'])
 
+        # Deal with manual extraction
+        row = self.fitstbl[frames[0]]
+        manual_obj = ManualExtractionObj.by_fitstbl_input(
+            row['filename'], row['manual']) if len(row['manual'].strip()) > 0 else None
+
         # Instantiate Reduce object
         # Required for pypeline specific object
         # At instantiaton, the fullmask in self.sciImg is modified
-        self.redux = reduce.Reduce.get_instance(sciImg, self.spectrograph,
-                                                self.par, self.caliBrate,
-                                                self.objtype,
-                                                ir_redux=self.ir_redux,
-                                                find_negative=self.find_negative,
-                                                std_redux=self.std_redux,
-                                                setup=self.setup,
-                                                show=self.show,
-                                                det=det, binning=self.binning,
-                                                basename=self.basename)
+        objFind = find_objects.FindObjects.get_instance(sciImg, self.spectrograph,
+                                                        self.par, self.caliBrate,
+                                                        self.objtype,
+                                                        bkg_redux=self.bkg_redux,
+                                                        manual=manual_obj,
+                                                        find_negative=self.find_negative,
+                                                        std_redux=self.std_redux,
+                                                        show=self.show,
+                                                        basename=self.basename)
         # Show?
         if self.show:
-            self.redux.show('image', image=sciImg.image, chname='processed',
+            objFind.show('image', image=sciImg.image, chname='processed',
                             slits=True, clear=True)
 
         # Do it
-        global_sky, sobjs_obj, skymask = self.redux.run_objfind(std_trace=std_trace, show_peaks=self.show)
-        return global_sky, sobjs_obj, skymask, sciImg
+        initial_sky, sobjs_obj = objFind.run(std_trace=std_trace, show_peaks=self.show)
+        # Return
+        return initial_sky, sobjs_obj, sciImg, objFind
 
-    def extract_one(self, frames, det, sciImg, global_sky, sobjs_obj, skymask):
+    def extract_one(self, frames, det, sciImg, objFind, initial_sky, sobjs_obj):
         """
         Extract Objects in a single exposure/detector pair
 
@@ -828,12 +830,12 @@ class PypeIt:
             sciImg (:class:`PypeItImage`):
                 Data container that holds a single image from a
                 single detector its related images (e.g. ivar, mask)
-            global_sky (`numpy.ndarray`_):
+            objFind : :class:`~pypeit.find_objects.FindObjects`
+                Object finding object
+            initial_sky (`numpy.ndarray`_):
                 Initial global sky model
             sobjs_obj (:class:`pypeit.specobjs.SpecObjs`):
                 List of objects found during `run_objfind`
-            skymask (`numpy.ndarray`_):
-               Boolean image indicating which pixels are useful for global sky subtraction
 
         Returns:
             tuple: Returns six `numpy.ndarray`_ objects and a
@@ -850,31 +852,44 @@ class PypeIt:
         # Is this a standard star?
         self.std_redux = 'standard' in self.objtype
 
+        # Update the skymask
+        skymask = objFind.create_skymask(sobjs_obj)
+
+        # Update the global sky
+        if 'standard' in self.fitstbl['frametype'][frames[0]] or \
+                self.par['reduce']['findobj']['skip_final_global'] or \
+                self.par['reduce']['skysub']['load_mask'] or \
+                self.par['reduce']['skysub']['user_regions'] is not None:
+            final_global_sky = initial_sky
+        else:
+            final_global_sky = objFind.global_skysub(previous_sky=initial_sky, skymask=skymask, show=self.show)
+
         msgs.info("Extraction begins for {} on det={}".format(self.basename, det))
 
         # Instantiate Reduce object
         # Required for pypeline specific object
         # At instantiaton, the fullmask in self.sciImg is modified
-        self.redux = reduce.Reduce.get_instance(sciImg, self.spectrograph,
-                                                self.par, self.caliBrate,
-                                                self.objtype,
-                                                ir_redux=self.ir_redux,
-                                                find_negative=self.find_negative,
-                                                std_redux=self.std_redux,
-                                                setup=self.setup,
-                                                show=self.show,
-                                                det=det, binning=self.binning,
-                                                basename=self.basename)
+        # TODO Are we repeating steps in the init for FindObjects and Extract??
+        self.redux = extraction.Extract.get_instance(
+            sciImg, sobjs_obj, self.spectrograph, 
+            self.par, self.caliBrate, self.objtype, 
+            bkg_redux=self.bkg_redux, 
+            find_negative=self.find_negative,
+            std_redux=self.std_redux,
+            show=self.show,
+            basename=self.basename)
 
         if not self.par['reduce']['extraction']['skip_extraction']:
-            skymodel, objmodel, ivarmodel, outmask, sobjs, scaleImg, waveImg, tilts = self.redux.run_extraction(
-                global_sky, sobjs_obj, skymask, ra=self.fitstbl["ra"][frames[0]], dec=self.fitstbl["dec"][frames[0]],
-                obstime=self.obstime)
+            # DP: what should we put here for return_negative?
+            skymodel, objmodel, ivarmodel, outmask, sobjs, scaleImg, waveImg, \
+                tilts = self.redux.run_extraction(final_global_sky, sobjs_obj, ra=self.fitstbl["ra"][frames[0]],
+                                                  dec=self.fitstbl["dec"][frames[0]], obstime=self.obstime,
+                                                  return_negative=False)
         else:
             # Although exrtaction is not performed, still need to prepare some masks and the tilts
-            self.redux.prepare_extraction(global_sky)
+            self.redux.prepare_extraction(final_global_sky)
             # Since the extraction was not performed, fill the arrays with the best available information
-            skymodel = self.redux.initial_sky
+            skymodel = self.redux.global_sky
             objmodel = np.zeros_like(self.redux.sciImg.image)
             ivarmodel = np.copy(self.redux.sciImg.ivar)
             outmask = self.redux.sciImg.fullmask
@@ -895,9 +910,13 @@ class PypeIt:
         spec_flex_table['spat_id'] = self.caliBrate.slits.spat_id
         spec_flex_table['sci_spec_flexure'] = self.redux.slitshift
 
+        # pull out maskdef_designtab from caliBrate.slits
+        maskdef_designtab = self.caliBrate.slits.maskdef_designtab
+        slits = copy.deepcopy(self.caliBrate.slits)
+        slits.maskdef_designtab = None
+
         # Construct the Spec2DObj
-        spec2DObj = spec2dobj.Spec2DObj(det=self.det,
-                                        sciimg=sciImg.image,
+        spec2DObj = spec2dobj.Spec2DObj(sciimg=sciImg.image,
                                         ivarraw=sciImg.ivar,
                                         skymodel=skymodel,
                                         objmodel=objmodel,
@@ -911,9 +930,13 @@ class PypeIt:
                                         vel_corr=self.redux.vel_corr,
                                         vel_type=self.par['calibrations']['wavelengths']['refframe'],
                                         tilts=tilts,
-                                        slits=copy.deepcopy(self.caliBrate.slits))
+                                        slits=slits,
+                                        maskdef_designtab=maskdef_designtab)
         spec2DObj.process_steps = sciImg.process_steps
 
+        # QA
+        spec2DObj.gen_qa()
+
         # Return
         return spec2DObj, sobjs
 
@@ -949,13 +972,27 @@ class PypeIt:
         if not os.path.isdir(self.science_path):
             os.makedirs(self.science_path)
 
+        # NOTE: There are some gymnastics here to keep from altering
+        # self.par['rdx']['detnum'].  I.e., I can't just set update_det =
+        # self.par['rdx']['detnum'] because that can alter the latter if I don't
+        # deepcopy it...
+        if self.par['rdx']['detnum'] is None:
+            update_det = None
+        elif isinstance(self.par['rdx']['detnum'], list):
+            update_det = [self.spectrograph.allowed_mosaics.index(d)+1 
+                            if isinstance(d, tuple) else d for d in self.par['rdx']['detnum']]
+        else:
+            update_det = self.par['rdx']['detnum']
+
         subheader = self.spectrograph.subheader_for_spec(row_fitstbl, head2d)
         # 1D spectra
         if all_specobjs.nobj > 0:
             # Spectra
             outfile1d = os.path.join(self.science_path, 'spec1d_{:s}.fits'.format(basename))
+            # TODO
+            #embed(header='deal with the following for maskIDs;  713 of pypeit')
             all_specobjs.write_to_fits(subheader, outfile1d,
-                                       update_det=self.par['rdx']['detnum'],
+                                       update_det=update_det,
                                        slitspatnum=self.par['rdx']['slitspatnum'],
                                        history=history)
             # Info
@@ -977,8 +1014,11 @@ class PypeIt:
                                                master_dir=self.caliBrate.master_dir,
                                                subheader=subheader,
                                                history=history)
+
         # Write
-        all_spec2d.write_to_fits(outfile2d, pri_hdr=pri_hdr, update_det=self.par['rdx']['detnum'])
+        all_spec2d.write_to_fits(outfile2d, pri_hdr=pri_hdr,
+                                 update_det=update_det,
+                                 slitspatnum=self.par['rdx']['slitspatnum'])
 
 
     def msgs_reset(self):
