@@ -8,7 +8,6 @@ from IPython import embed
 
 from pypeit.scripts import scriptbase
 
-
 class Identify(scriptbase.ScriptBase):
 
     @classmethod
@@ -45,6 +44,7 @@ class Identify(scriptbase.ScriptBase):
 
         import numpy as np
         
+        from pypeit import msgs
         from pypeit import masterframe
         from pypeit.spectrographs.util import load_spectrograph
         from pypeit.core.gui.identify import Identify
@@ -54,33 +54,20 @@ class Identify(scriptbase.ScriptBase):
         from pypeit.images.buildimage import ArcImage
 
         # Load the MasterArc file
-        if os.path.exists(args.arc_file):
-            arcfil = args.arc_file
-        else:
-            try:
-                arcfil = "Masters/{0:s}".format(args.arc_file)
-            except FileNotFoundError:
-                print("Could not find MasterArc file.")
-                sys.exit()
-        msarc = ArcImage.from_file(arcfil)
-
-        mdir = msarc.head0['MSTRDIR']
-        mkey = msarc.head0['MSTRKEY']
+        msarc = ArcImage.from_file(args.arc_file)
 
         # Load the spectrograph
-        specname = msarc.head0['PYP_SPEC']
-        spec = load_spectrograph(specname)
+        spec = load_spectrograph(msarc.PYP_SPEC)
         par = spec.default_pypeit_par()['calibrations']['wavelengths']
 
         # Get the lamp list
         if args.lamps is None:
-            lamplist = par['lamps']
-            if lamplist is None:
-                print("ERROR :: Cannot determine the lamps")
-                sys.exit()
+            lamps = par['lamps']
+            if lamps is None:
+                msgs.error('Cannot determine the lamps')
         else:
-            lamplist = args.lamps.split(",")
-        par['lamps'] = lamplist
+            lamps = args.lamps.split(",")
+        par['lamps'] = lamps
 
         # Load the slits
         slits = slittrace.SlitTraceSet.from_file(args.slits_file)
@@ -88,19 +75,22 @@ class Identify(scriptbase.ScriptBase):
         slits.mask = slits.mask_init
 
         # Check if a solution exists
-        solnname = os.path.join(mdir, masterframe.construct_file_name(WaveCalib, mkey))
+        solnname = masterframe.construct_file_name(WaveCalib, msarc.master_key,
+                                                   master_dir=msarc.master_dir)
         wv_calib = waveio.load_wavelength_calibration(solnname) \
                         if os.path.exists(solnname) and args.solution else None
 
-        # Load the MasterFrame (if it exists and is desired)?
-        wavecal = BuildWaveCalib(msarc, slits, spec, par, binspectral=slits.binspec, det=args.det,
-                            master_key=mkey, msbpm=msarc.fullmask)
+        # Load the MasterFrame (if it exists and is desired).  Bad-pixel mask
+        # set to any flagged pixel in MasterArc.
+        wavecal = BuildWaveCalib(msarc, slits, spec, par, lamps, binspectral=slits.binspec,
+                                 det=args.det, master_key=msarc.master_key,
+                                 msbpm=msarc.select_flag())
         arccen, arc_maskslit = wavecal.extract_arcs(slitIDs=[args.slit])
 
         # Launch the identify window
-        arcfitter = Identify.initialise(arccen, slits, slit=int(args.slit), par=par,
+        arcfitter = Identify.initialise(arccen, lamps, slits, slit=int(args.slit), par=par,
                                         wv_calib_all=wv_calib, wavelim=[args.wmin, args.wmax],
-                                        nonlinear_counts=spec.nonlinear_counts(msarc.detector),
+                                        nonlinear_counts=msarc.detector.nonlinear_counts(),
                                         pxtoler=args.pixtol, test=args.test, fwhm=args.fwhm)
         # Testing?
         if args.test:
@@ -113,12 +103,13 @@ class Identify(scriptbase.ScriptBase):
         if 'WaveFit' in arcfitter._fitdict.keys():
             waveCalib = WaveCalib(nslits=1, wv_fits=np.atleast_1d(arcfitter._fitdict['WaveFit']),
                                   arc_spectra=np.atleast_2d(arcfitter.specdata).T,
-                                  spat_ids=np.atleast_1d(arcfitter._slit), PYP_SPEC=specname)
+                                  spat_ids=np.atleast_1d(int(arcfitter._spatid)), PYP_SPEC=msarc.PYP_SPEC,
+                                  lamps=','.join(lamps))
         else:
             waveCalib = None
 
         # Ask the user if they wish to store the result in PypeIt calibrations
-        arcfitter.store_solution(final_fit, mdir, slits.binspec,
+        arcfitter.store_solution(final_fit, slits.binspec,
                                  wvcalib=waveCalib,
                                  rmstol=args.rmstol,
                                  force_save=args.force_save)
